package de.hdm_stuttgart.battlearena.Persistance.Classes;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import de.hdm_stuttgart.battlearena.Exceptions.DatabaseError;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.FileReader;
import java.io.FileWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;

public class GsonHandler {
    Gson gson = new GsonBuilder().setPrettyPrinting().create();

    private static final Logger log = LogManager.getLogger(GsonHandler.class);
    Type mapDataType = new TypeToken<ArrayList<MapData>>(){}.getType();

    public ArrayList<MapData> loadMaps(String filePath) throws DatabaseError{
        try (FileReader reader = new FileReader(filePath)) {
            ArrayList<MapData> maps = new ArrayList<MapData>();
            maps = gson.fromJson(reader, mapDataType);
            log.info("GSON - Maps successfully loaded from JSON");
            return maps;
        } catch (Exception e) {
            log.info(e);
            log.info("GSON - Loading Maps from JSON failed");
            throw new DatabaseError("Error Loading Maps!");
        }
    }

    public void saveMaps(ArrayList<MapData> maps, String filePath) throws DatabaseError{
        try (FileWriter writer = new FileWriter(filePath)) {
            gson.toJson(maps, writer);
            log.info("GSON - Maps successfully saved to JSON");
        } catch (
                Exception e) {
            log.info(e);
            log.info("GSON - Saving Maps to JSON failed");
            throw new DatabaseError("Error Loading Maps!");
        }
    }
}
