package de.hdm_stuttgart.battlearena.Persistance.Classes;

import de.hdm_stuttgart.battlearena.Exceptions.DatabaseError;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.sql.*;
import java.util.ArrayList;
import java.util.Properties;

import oracle.jdbc.pool.OracleDataSource;
import oracle.jdbc.OracleConnection;

public class OracleDB implements IDataBase {

    private static final Logger log = LogManager.getLogger(OracleDB.class);
    @Override
    public Connection connect() throws DatabaseError{
        try {
            OracleDataSource ods = new OracleDataSource();
            ods.setURL("jdbc:oracle:thin:@(description= (retry_count=20)(retry_delay=3)(address=(protocol=tcps)(port=1521)(host=adb.eu-frankfurt-1.oraclecloud.com))(connect_data=(service_name=g093caf2cf1fea4_battlearena_high.adb.oraclecloud.com))(security=(ssl_server_dn_match=yes)))");
            ods.setUser("battlearenaplayer");
            ods.setPassword("jo3+++w3rw+s##AA");
            Connection conn = ods.getConnection();

            log.info("Connecting to the database!");

            return conn;
        }
        catch(Exception e){
            log.error(e);
            throw new DatabaseError("SQL connection error");
        }
    }

    @Override
    public ArrayList<MapData> getCoreMaps() throws DatabaseError{
        try(Connection connection = connect()) {
            ArrayList<MapData> newMaps = new ArrayList<MapData>();
            String sql = "SELECT * FROM battlearenadata.coremaps";
            PreparedStatement stmt = connection.prepareStatement(sql);

            log.info("Sending SQL statement");
            ResultSet rs = stmt.executeQuery();

            while(rs.next()){
             newMaps.add(new MapData(rs.getString("map_id"), rs.getString("map_name"), rs.getInt("map_width"), rs.getInt("map_height"), rs.getString("map_data")));
            }
            rs.close();
            stmt.close();
            log.info("Coremaps retrieved successfully");
            return newMaps;
        }
        catch(Exception e){
            log.error(e);
            throw new DatabaseError("Error retrieving coremaps");
        }
    }

    public ArrayList<String> getCommunityMapsList() throws DatabaseError{
        try(Connection connection = connect()) {
            ArrayList<String> tempList = new ArrayList<String>();
            String sql = "SELECT map_name FROM battlearenadata.communitymaps";
            PreparedStatement stmt = connection.prepareStatement(sql);

            log.info("Sending SQL statement");
            ResultSet rs = stmt.executeQuery();

            while(rs.next()){
                tempList.add(rs.getString("map_name"));
            }
            rs.close();
            stmt.close();
            log.info("Community map names retrieved successfully");
            return tempList;
        }
        catch(Exception e){
            log.error(e);
            throw new DatabaseError("Error retrieving community map names");
        }
    }

    public MapData getCommunityMapByID(String mapID) throws DatabaseError{
        try(Connection connection = connect()) {
            String sql = "SELECT * FROM battlearenadata.communitymaps WHERE map_ID = ?";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, mapID);

            log.info("Sending SQL statement");
            ResultSet rs = stmt.executeQuery();

            MapData mapChosen = new MapData(rs.getString("map_id"), rs.getString("map_name"), rs.getInt("map_width"), rs.getInt("map_height"), rs.getString("map_data"));

            rs.close();
            stmt.close();
            log.info("Community map retrieved successfully");
            return mapChosen;
        }
        catch(Exception e){
            log.error(e);
            throw new DatabaseError("Error retrieving community map");
        }
    }

    public void uploadCommunityMapByID(MapData map) throws DatabaseError{
        try(Connection connection = connect()) {
            String sql = "INSERT INTO battlearenadata.communitymaps (map_id, map_name, map_width, map_height, map_data) VALUES (?, ?, ?, ?, ?)";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, map.getMapID());
            stmt.setString(2, map.getMapName());
            stmt.setInt(3, map.getMapWidth());
            stmt.setInt(4, map.getMapHeight());
            stmt.setString(5, map.getMapData());

            log.info("Sending SQL statement");
            stmt.executeQuery();

            stmt.close();
            log.info("Community Map retrieved successfully");
        }
        catch(Exception e){
            log.error(e);
            throw new DatabaseError("Error uploading created community map");
        }
    }

/*
    public void updateCommunityStats(Persistence persistence){
         //Update the commulative Stats
    }
*/

}
